@@ -2,7 +2,7 @@
    esmtp.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 #include "extern.h"
 
 #ifdef HAVE_TLS
- #include <gcrypt.h>
+# include <gcrypt.h>
 #endif /* HAVE_TLS */
 
 static char *b64encode(char *);
@@ -47,7 +47,7 @@ esmtp_auth(void *sd_server, char *reply)
 	char tmp[LINEBUFFER+1];
 	memset(tmp, 0, LINEBUFFER + 1);
 
-	#if defined(HAVE_TLS) || defined(HAVE_SSL)
+#if defined(HAVE_TLS) || defined(HAVE_SSL)
 	if (strstr(reply, "CRAM-MD5")) {
 		int i;
 		unsigned char digest[16];
@@ -76,20 +76,11 @@ esmtp_auth(void *sd_server, char *reply)
 		for (i = 0; i < 16; i++)
 			sprintf(ascii_digest + 2 * i, "%02x", digest[i]);
 
-		#ifdef HAVE_SNPRINTF
 		snprintf(tmp, LINEBUFFER,
-		#else
-		sprintf(tmp,
-		#endif /* HAVE_SNPRINTF */
 			"%s %s", session.mta_username, ascii_digest);
 
 		p = b64encode(tmp);
-		#ifdef HAVE_SNPRINTF
-		snprintf(tmp, LINEBUFFER,
-		#else
-		sprintf(tmp,
-		#endif /* HAVE_SNPRINTF */
-			"%s"CRLF, p);
+		snprintf(tmp, LINEBUFFER, "%s"CRLF, p);
 		xfree(p);
 
 		swrite(CLIENT, sd_server, tmp);
@@ -100,7 +91,7 @@ esmtp_auth(void *sd_server, char *reply)
 		}
 	}
 	else
-	#endif /* HAVE_TLS or HAVE_SSL */
+#endif /* HAVE_TLS or HAVE_SSL */
 	if (strstr(reply, "LOGIN") && (topt & T_SSL_FINISHED)) {
 		info(VERBOSE, _("Using the ESMTP LOGIN authentication..."));
 		swrite(CLIENT, sd_server, "AUTH LOGIN"CRLF);
@@ -119,12 +110,7 @@ esmtp_auth(void *sd_server, char *reply)
 		info(DEBUG, _("Challenge decoded: %s"), b64buf);
 
 		p = b64encode(session.mta_username);
-		#ifdef HAVE_SNPRINTF
-		snprintf(tmp, LINEBUFFER,
-		#else
-		sprintf(tmp,
-		#endif /* HAVE_SNPRINTF */
-			"%s"CRLF, p);
+		snprintf(tmp, LINEBUFFER, "%s"CRLF, p);
 		xfree(p);
 		swrite(CLIENT, sd_server, tmp);
 		get_response_smtp(CLIENT, sd_server, tmp, LINEBUFFER);
@@ -135,12 +121,7 @@ esmtp_auth(void *sd_server, char *reply)
 		info(DEBUG, _("Challenge decoded: %s"), b64buf);
 
 		p = b64encode(session.mta_password);
-		#ifdef HAVE_SNPRINTF
-		snprintf(tmp, LINEBUFFER,
-		#else
-		sprintf(tmp,
-		#endif /* HAVE_SNPRINTF */
-			"%s"CRLF, p);
+		snprintf(tmp, LINEBUFFER, "%s"CRLF, p);
 		xfree(p);
 		swrite(CLIENT, sd_server, tmp);
 
