@@ -124,8 +124,8 @@ make_sockets(int fd[2])
 	return 0;
 }
 
-static void
-sig_local(int code)
+void
+cleanup_children()
 {
 	pid_t pid;
 	int status;
@@ -133,7 +133,20 @@ sig_local(int code)
 	while ((pid = waitpid(-1, &status, WNOHANG)) > 0)
 		info(VERBOSE, _("Local program [%lu] finished."),
 		     (unsigned long) pid);
-	return;
+}
+
+static RETSIGTYPE
+sig_local(int code)
+{
+	/* EMPTY FUNCTION */
+	/* Notice: This signal handler is installed only by
+	   make_local_connection, that is used to launch the local
+	   mailer. After the mailer exits, the main code will read
+	   the return code from the mailer's outgoing pipe. If
+	   waitpid() is called before this happens, the pipe will be
+	   broken and the read will fail. Therefore, do not call waitpid()
+	   from the handler. This should be done after smtp_session()
+	   by calling cleanup_children() */
 }
 
 int
@@ -149,8 +162,7 @@ make_local_connection(char *exec_path, char **exec_args)
 	memset(args, 0, LINEBUFFER + 1);
 	pargs = exec_args;
 	pargs++;
-	while (*pargs)
-	{
+	while (*pargs) {
 		strncat(args, *pargs, LINEBUFFER - strlen(args));
 		strncat(args, " ", LINEBUFFER - strlen(args));
 		pargs++;
@@ -219,20 +231,20 @@ external_program(int *rs, char *path, char *src, char *dst, int dstsize)
 	}
 	
 	args = gen_execargs(a);
-	ret = exec_argv(rs, args, src, dst, dstsize);
+	ret = exec_argv(rs, p, args, src, dst, dstsize);
 	xfree_pptr(args);
 	return ret;
 }
 
 char *
-exec_argv(int *rs, char **argv, char *src, char *dst, int dstsize)
+exec_argv(int *rs, char *path, char **argv, char *src, char *dst, int dstsize)
 {
 	int status;
 	int fd;
 	int n;
 	char *buf;
 	
-	fd = make_local_connection(argv[0], argv);
+	fd = make_local_connection(path ? path : argv[0], argv);
 	if (fd == -1) {
 		*rs = -1;
 		return 0;
