@@ -28,8 +28,8 @@ extern const char copyright[];
 struct options_struct {
  int termlevel;
  int uloglevel;
- char *slogfile;
  char *ulogfile;
+ char *tracefile;
 #ifdef WITH_GUILE
  char *guile_logfile;
 #endif /* WITH_GUILE */
