@@ -31,7 +31,8 @@
 
 struct gpg_struct {
 	int inited;
-	char *keys;
+	char *sign_keys;
+	char *encryption_keys;
 	char *passphrase;
 };
 
@@ -39,7 +40,8 @@ static struct gpg_struct gpg;
 
 static int gpgme_init(void);
 static char *gpg_sign(char *);
-static char *gpg_encrypt_to_users(char *);
+static char *gpg_encrypt(char *);
+static char *gpg_sign_encrypt(char *);
 static void gpgme_debug_info(GpgmeCtx);
 
 #define EXTRA_GPG_BUF 4096
@@ -58,7 +60,6 @@ gpgme_debug_info(GpgmeCtx ctx)
 		puts(s);
 		free(s);
 	}
-	return;
 }
 
 #define GPGME_REQ_VERSION "0.3.12" /* GPGME 0.3.12 or later */
@@ -98,9 +99,8 @@ gpg_sign(char *gpg_data)
 	GpgmeError err = 0;
 	GpgmeData in, out;
 	GpgmeKey key;
-	
 	char buf[256];
-	char *signed_data;
+	char *p, *signed_data;
 	int size;
 	size_t nread;
 
@@ -109,19 +109,23 @@ gpg_sign(char *gpg_data)
 	memset(buf, 0, sizeof(buf));
 	fail_if_err(gpgme_new(&ctx));
 
-	if (gpg.keys) {
-		err = gpgme_op_keylist_start(ctx, gpg.keys, 0);
+	if (gpg.sign_keys) {
+		err = gpgme_op_keylist_start(ctx, gpg.sign_keys, 0);
 		if (!err) {
 			while ((err = gpgme_op_keylist_next(ctx, &key)) == 0)
 				err = gpgme_signers_add(ctx, key);
 		}
 		if (err && err != GPGME_EOF) {
-			anubis_error(HARD, _("GPGME: cannot list keys: %s"),
+			anubis_error(HARD, _("GPGME: Cannot list keys: %s"),
 				     gpgme_strerror(err));
 			topt |= T_ERROR;
 		}
 	}
-	gpgme_set_passphrase_cb(ctx, (GpgmePassphraseCb)passphrase_cb, 0);
+
+	p = getenv("GPG_AGENT_INFO");
+	if (!(p && strchr (p, ':')))
+		gpgme_set_passphrase_cb(ctx,
+					(GpgmePassphraseCb)passphrase_cb, 0);
 	gpgme_set_textmode(ctx, 1);
 	gpgme_set_armor(ctx, 1);
 
@@ -164,7 +168,7 @@ gpg_sign(char *gpg_data)
 }
 
 static char *
-gpg_encrypt_to_users(char *gpg_data)
+gpg_encrypt(char *gpg_data)
 {
 	GpgmeCtx ctx;
 	GpgmeError err = 0;
@@ -184,21 +188,23 @@ gpg_encrypt_to_users(char *gpg_data)
 
 	fail_if_err(gpgme_new(&ctx));
 	gpgme_set_armor(ctx, 1);
+
 	fail_if_err(gpgme_data_new_from_mem(&in, gpg_data, strlen(gpg_data), 0));
 	fail_if_err(gpgme_data_new(&out));
 	fail_if_err(gpgme_recipients_new(&rset));
 
-	len = strlen(gpg.keys);
+	len = strlen(gpg.encryption_keys);
 	for (i = 0; i <= len; i++)
 	{
-		if (gpg.keys[i] == ',') { /* comma found, so add KEY-ID */
+		if (gpg.encryption_keys[i] == ','
+		    || gpg.encryption_keys[i] == '\0') {
 			fail_if_err(gpgme_recipients_add_name_with_validity(rset,
 				current_key, GPGME_VALIDITY_FULL));
 			memset(current_key, 0, sizeof(current_key));
 			j = 0;
 		}
-		else /* it is not a comma, so add char to KEY_ID string */
-			current_key[j++] = gpg.keys[i];
+		else
+			current_key[j++] = gpg.encryption_keys[i];
 	}
 	fail_if_err(gpgme_op_encrypt(ctx, rset, in, out));
 	fail_if_err(gpgme_data_rewind(out));
@@ -238,6 +244,101 @@ gpg_encrypt_to_users(char *gpg_data)
 	return encrypted_data;
 }
 
+static char *
+gpg_sign_encrypt(char *gpg_data)
+{
+	GpgmeCtx ctx;
+	GpgmeError err = 0;
+	GpgmeData in, out;
+	GpgmeKey key;
+	GpgmeRecipients rset;
+	char current_key[100];
+	char buf[256];
+	char *p, *se_data; /* Signed-Encrypted Data */
+	int i, j = 0;
+	int len, size;
+	size_t nread;
+
+	size = strlen(gpg_data) + EXTRA_GPG_BUF;
+	se_data = (char *)xmalloc(size);
+	memset(buf, 0, sizeof(buf));
+	memset(current_key, 0, sizeof(current_key));
+
+	fail_if_err(gpgme_new(&ctx));
+
+	if (gpg.sign_keys) {
+		err = gpgme_op_keylist_start(ctx, gpg.sign_keys, 0);
+		if (!err) {
+			while ((err = gpgme_op_keylist_next(ctx, &key)) == 0)
+				err = gpgme_signers_add(ctx, key);
+		}
+		if (err && err != GPGME_EOF) {
+			anubis_error(HARD, _("GPGME: Cannot list keys: %s"),
+				     gpgme_strerror(err));
+			topt |= T_ERROR;
+		}
+	}
+
+	p = getenv("GPG_AGENT_INFO");
+	if (!(p && strchr (p, ':')))
+		gpgme_set_passphrase_cb(ctx,
+					(GpgmePassphraseCb)passphrase_cb, 0);
+	gpgme_set_armor(ctx, 1);
+
+	fail_if_err(gpgme_data_new_from_mem(&in, gpg_data, strlen(gpg_data), 0));
+	fail_if_err(gpgme_data_new(&out));
+	fail_if_err(gpgme_recipients_new(&rset));
+
+	len = strlen(gpg.encryption_keys);
+	for (i = 0; i <= len; i++)
+	{
+		if (gpg.encryption_keys[i] == ','
+		    || gpg.encryption_keys[i] == '\0') {
+			fail_if_err(gpgme_recipients_add_name_with_validity(rset,
+				current_key, GPGME_VALIDITY_FULL));
+			memset(current_key, 0, sizeof(current_key));
+			j = 0;
+		}
+		else
+			current_key[j++] = gpg.encryption_keys[i];
+	}
+	fail_if_err(gpgme_op_encrypt_sign(ctx, rset, in, out));
+	fail_if_err(gpgme_data_rewind(out));
+
+	if (options.termlevel == DEBUG)
+		gpgme_debug_info(ctx);
+
+	if (topt & T_ERROR) {
+		gpgme_recipients_release(rset);
+		gpgme_release(ctx);
+		free(se_data);
+		return 0;
+	}
+
+	while (!(err = gpgme_data_read(out, buf, sizeof(buf), &nread)))
+	{
+		if (size > nread) {
+			strncat(se_data, buf, nread);
+			size -= nread;
+		}
+		else {
+			size = EXTRA_GPG_BUF;
+			se_data = (char *)xrealloc((char *)se_data,
+				strlen(se_data) + size);
+			strncat(se_data, buf, nread);
+			size -= nread;
+		}
+		memset(buf, 0, sizeof(buf));
+	}
+	if (err != GPGME_EOF)
+		fail_if_err(err);
+
+	gpgme_recipients_release(rset);
+	gpgme_data_release(in);
+	gpgme_data_release(out);
+	gpgme_release(ctx);
+	return se_data;
+}
 
 void
 gpg_proc(MESSAGE *msg, char *(*proc)(char *input))
@@ -263,8 +364,6 @@ gpg_proc(MESSAGE *msg, char *(*proc)(char *input))
 #if defined(HAVE_SETENV) || defined(HAVE_PUTENV)
 	setenv("HOME", homedir_s, 1);
 #endif /* HAVE_SETENV or HAVE_PUTENV */
-
-	return;
 }
 
 void
@@ -274,13 +373,15 @@ gpg_free(void)
 		memset(gpg.passphrase, 0, strlen(gpg.passphrase));
 		xfree(gpg.passphrase);
 	}
-	xfree(gpg.keys);
+	xfree(gpg.sign_keys);
+	xfree(gpg.encryption_keys);
 }
 
 #define KW_GPG_PASSPHRASE         1
 #define KW_GPG_ENCRYPT            2
 #define KW_GPG_SIGN               3
-#define KW_GPG_HOME               4
+#define KW_GPG_SIGN_ENCRYPT       4
+#define KW_GPG_HOME               5
 
 int
 gpg_parser(int method, int key, LIST *arglist,
@@ -294,29 +395,49 @@ gpg_parser(int method, int key, LIST *arglist,
 			xfree(gpg.passphrase);
 		}
 		gpg.passphrase = strdup(arg);
+		arg = NULL;
 		break;
 		
 	case KW_GPG_ENCRYPT:
-		xfree(gpg.keys);
-		gpg.keys = allocbuf(arg, 0);
-		gpg.keys = xrealloc(gpg.keys, strlen(gpg.keys) + 2);
-		strcat(gpg.keys, ",");
+		xfree(gpg.encryption_keys);
+		gpg.encryption_keys = allocbuf(arg, 0);
 		if (gpg.inited == 0 && gpgme_init())
 			break;
-		gpg_proc(msg, gpg_encrypt_to_users);
+		gpg_proc(msg, gpg_encrypt);
 		break;
 		
 	case KW_GPG_SIGN:
 		if (strcasecmp(arg, "no")) {
-			xfree(gpg.keys);
-			if (strcasecmp(arg, "yes")) 
-				gpg.keys = strdup(arg);
-			if (gpg.inited == 0 && gpgme_init()) 
+			xfree(gpg.sign_keys);
+			if (strcasecmp(arg, "default") && strcasecmp(arg, "yes"))
+				gpg.sign_keys = strdup(arg);
+			if (gpg.inited == 0 && gpgme_init())
 				break;
 			gpg_proc(msg, gpg_sign);
 		}
 		break;
-		
+
+	case KW_GPG_SIGN_ENCRYPT: {
+		char *p = strchr(arg, ':');
+		xfree(gpg.encryption_keys);
+		if (p) {
+			p++;
+			if (strcasecmp(p, "default") && strcasecmp(p, "yes")) {
+				xfree(gpg.sign_keys);
+				gpg.sign_keys = allocbuf(p, 0);
+			}
+			*--p = '\0';
+			gpg.encryption_keys = allocbuf(arg, 0);
+		}
+		else
+			gpg.encryption_keys = allocbuf(arg, 0);
+
+		if (gpg.inited == 0 && gpgme_init())
+			break;
+		gpg_proc(msg, gpg_sign_encrypt);
+	}
+		break;
+
 	case KW_GPG_HOME:
 		setenv("GNUPGHOME", arg, 1);
 		break;
@@ -329,10 +450,12 @@ gpg_parser(int method, int key, LIST *arglist,
 
 
 struct rc_kwdef gpg_kw[] = {
-	{ "gpg-passphrase",  KW_GPG_PASSPHRASE },          
-	{ "gpg-encrypt",     KW_GPG_ENCRYPT },             
-	{ "gpg-sign",        KW_GPG_SIGN },                
-	{ "gpg-home",        KW_GPG_HOME },
+	{ "gpg-passphrase",    KW_GPG_PASSPHRASE },
+	{ "gpg-encrypt",       KW_GPG_ENCRYPT },
+	{ "gpg-sign",          KW_GPG_SIGN },
+	{ "gpg-sign-encrypt",  KW_GPG_SIGN_ENCRYPT },
+	{ "gpg-se",            KW_GPG_SIGN_ENCRYPT },
+	{ "gpg-home",          KW_GPG_HOME },
 	{ NULL },
 };
 
