@@ -236,6 +236,9 @@
 #define T_CHECK_CONFIG      0x00400000
 #define T_RELAX_PERM_CHECK  0x00800000
 #define T_ENTIRE_BODY       0x01000000
+#define T_SMTP_ERROR_CODES  0x02000000
+#define T_TRACEFILE_SYS     0x04000000
+#define T_TRACEFILE_USR     0x08000000
 
 /* Regexp modifiers */
 /* Basic types */
@@ -273,13 +276,13 @@ void *xmalloc(int);
 void *xrealloc(void *, int);
 char *allocbuf(char *, int);
 #ifndef HAVE_STRDUP
- char *strdup(const char *);
+char *strdup(const char *);
 #endif /* not HAVE_STRDUP */
 void free_pptr(char **);
 
 /* setenv.c */
 #if !defined(HAVE_SETENV) && defined(HAVE_PUTENV)
- int setenv(const char *, const char *, int);
+int setenv(const char *, const char *, int);
 #endif
 
 /* env.c */
@@ -293,14 +296,14 @@ int  check_filemode(char *);
 int  check_filename(char *, time_t *);
 
 /* errs.c */
-void anubis_error(int, char *, ...);
+void anubis_error(int, const char *, ...);
 void socket_error(const char *);
-void socks_error(char *);
+void socks_error(const char *);
 void hostname_error(char *);
 
 /* log.c */
-void mprintf(char *, ...);
-void info(int, char *, ...);
+void mprintf(const char *, ...);
+void info(int, const char *, ...);
 void filelog(char *, char *);
 
 /* net.c */
@@ -347,12 +350,14 @@ void message_free(MESSAGE *);
 char **gen_execargs(const char *);
 int  make_local_connection(char *, char **);
 char *external_program(int *, char *, char *, char *, int);
-char *exec_argv(int *, char **, char *, char *, int);
+char *exec_argv(int *, char *, char **, char *, char *, int);
+void cleanup_children();
 
 /* esmtp.c */
 void esmtp_auth(void *, char *);
 
 /* misc.c */
+int anubis_free_list_item(void *item, void *data);
 void assoc_free(ASSOC *);
 ASSOC *header_assoc(char *);
 void destroy_assoc_list(LIST **);
@@ -390,8 +395,8 @@ void print_usage(void);
 void print_config_options(void);
 
 /* quit.c */
-void sig_exit(int);
-void sig_timeout(int);
+RETSIGTYPE sig_exit(int);
+RETSIGTYPE sig_timeout(int);
 void free_mem(void);
 void quit(int);
 
@@ -404,8 +409,8 @@ void *start_ssl_server(int);
 
 /* gpg.c */
 #ifdef HAVE_GPG
- void gpg_free(void);
- void gpg_section_init(void);
+void gpg_free(void);
+void gpg_section_init(void);
 #endif /* HAVE_GPG */
 
 /* guile.c */
