@@ -66,7 +66,7 @@
 extern int rc_yylex(void);
 int rc_yyerror(char *s);
 
-static RC_SECTION *rc_section_create(char *, RC_STMT *);
+static RC_SECTION *rc_section_create(char *, size_t, RC_STMT *);
 static void rc_section_destroy(RC_SECTION **);
 static void rc_section_print(RC_SECTION *);
 static void rc_asgn_destroy(RC_ASGN *);
@@ -107,6 +107,11 @@ typedef union {
 		RC_REGEX *key;
 		char *string;
 	} msgpart;
+	RC_LOC loc;
+	struct {
+		size_t line;
+		char *name;
+	} begin_sec;
 	LIST *list;
 } YYSTYPE;
 #ifndef YYDEBUG
@@ -197,13 +202,13 @@ static const short rc_yyrhs[] = {    31,
 
 #if YYDEBUG != 0
 static const short rc_yyrline[] = { 0,
-   106,   111,   115,   126,   135,   139,   143,   149,   149,   156,
-   164,   167,   171,   184,   188,   189,   190,   191,   192,   193,
-   202,   220,   226,   231,   238,   241,   248,   257,   258,   262,
-   269,   276,   285,   286,   289,   295,   303,   308,   311,   316,
-   319,   327,   337,   344,   348,   354,   364,   370,   378,   382,
-   385,   391,   398,   401,   404,   412,   422,   429,   430,   433,
-   444,   455,   466,   477,   488,   504
+   112,   117,   121,   132,   141,   145,   149,   155,   155,   163,
+   173,   176,   180,   193,   197,   198,   199,   200,   201,   202,
+   211,   229,   235,   240,   247,   250,   257,   266,   267,   271,
+   278,   285,   294,   295,   298,   304,   312,   317,   320,   325,
+   328,   336,   346,   353,   357,   363,   373,   379,   387,   391,
+   394,   400,   407,   410,   413,   421,   431,   438,   439,   442,
+   453,   464,   475,   486,   497,   513
 };
 #endif
 
@@ -872,18 +877,18 @@ rc_yyreduce:
   switch (rc_yyn) {
 
 case 1:
-#line 107 "rcfile.y"
+#line 113 "rcfile.y"
 {
            ;
     break;}
 case 2:
-#line 112 "rcfile.y"
+#line 118 "rcfile.y"
 {
 		   rc_yyval.section = rc_section = rc_yyvsp[0].section;
 	   ;
     break;}
 case 3:
-#line 116 "rcfile.y"
+#line 122 "rcfile.y"
 {
 		   if (rc_yyvsp[0].section) {
 			   if (rc_section == NULL) {
@@ -896,7 +901,7 @@ case 3:
 	   ;
     break;}
 case 4:
-#line 127 "rcfile.y"
+#line 133 "rcfile.y"
 {
 		   lex_clear_state();
 		   error_sync_begin();
@@ -905,52 +910,55 @@ case 4:
 	   ;
     break;}
 case 5:
-#line 136 "rcfile.y"
+#line 142 "rcfile.y"
 {
 		   rc_yyval.section = NULL;
 	   ;
     break;}
 case 6:
-#line 140 "rcfile.y"
+#line 146 "rcfile.y"
 {
-		   rc_yyval.section = rc_section_create(rc_yyvsp[-2].string, rc_yyvsp[-1].stmtlist.head);
+		   rc_yyval.section = rc_section_create(rc_yyvsp[-2].begin_sec.name, rc_yyvsp[-2].begin_sec.line, rc_yyvsp[-1].stmtlist.head);
 	   ;
     break;}
 case 7:
-#line 144 "rcfile.y"
+#line 150 "rcfile.y"
 {
 		   rc_yyval.section = NULL;
 	   ;
     break;}
 case 8:
-#line 149 "rcfile.y"
+#line 155 "rcfile.y"
 { verbatim(); ;
     break;}
 case 9:
-#line 150 "rcfile.y"
+#line 156 "rcfile.y"
 {
-		   rc_yyval.string = rc_yyvsp[-1].string;
+		   rc_yyval.begin_sec.line = cfg_line_num - 1;
+		   rc_yyval.begin_sec.name = rc_yyvsp[-1].string;
 		   if (rc_section_lookup(rc_section, rc_yyvsp[-1].string)) 
 			   parse_error(_("Section %s already defined"), rc_yyvsp[-1].string);
 		   rc_secdef = anubis_find_section(rc_yyvsp[-1].string);
 	   ;
     break;}
 case 10:
-#line 157 "rcfile.y"
+#line 164 "rcfile.y"
 {
+		   rc_yyval.begin_sec.line = cfg_line_num - 1;
+		   rc_yyval.begin_sec.name = rc_yyvsp[-1].string;
 		   if (rc_section_lookup(rc_section, rc_yyvsp[-1].string)) 
 			   parse_error(_("Section %s already defined"), rc_yyvsp[-1].string);
 		   rc_secdef = anubis_find_section(rc_yyvsp[-1].string);
 	   ;
     break;}
 case 12:
-#line 168 "rcfile.y"
+#line 177 "rcfile.y"
 {
 		   rc_yyval.stmtlist.head = rc_yyval.stmtlist.tail = rc_yyvsp[0].stmt;
 	   ;
     break;}
 case 13:
-#line 172 "rcfile.y"
+#line 181 "rcfile.y"
 {
 		   if (rc_yyvsp[0].stmt) {
 			   if (rc_yyval.stmtlist.head == NULL) {
@@ -963,13 +971,13 @@ case 13:
 	   ;
     break;}
 case 14:
-#line 185 "rcfile.y"
+#line 194 "rcfile.y"
 {
 		   rc_yyval.stmt = NULL;
 	   ;
     break;}
 case 20:
-#line 194 "rcfile.y"
+#line 203 "rcfile.y"
 {
 		   lex_clear_state();
 		   rc_yyerrok;
@@ -978,7 +986,7 @@ case 20:
 	   ;
     break;}
 case 21:
-#line 203 "rcfile.y"
+#line 212 "rcfile.y"
 {
 		   if (!check_kw(rc_yyvsp[-1].string)) {
 			   parse_error(_("unknown keyword: %s"), rc_yyvsp[-1].string);
@@ -996,27 +1004,27 @@ case 21:
 	   ;
     break;}
 case 22:
-#line 221 "rcfile.y"
+#line 230 "rcfile.y"
 {
 		   verbatim();
 	   ;
     break;}
 case 23:
-#line 227 "rcfile.y"
+#line 236 "rcfile.y"
 {
 		   rc_yyval.list = list_create();
 		   list_append(rc_yyval.list, rc_yyvsp[0].string);
 	   ;
     break;}
 case 24:
-#line 232 "rcfile.y"
+#line 241 "rcfile.y"
 {
 		   list_append(rc_yyvsp[-1].list, rc_yyvsp[0].string);
 		   rc_yyval.list = rc_yyvsp[-1].list;
 	   ;
     break;}
 case 26:
-#line 242 "rcfile.y"
+#line 251 "rcfile.y"
 {
 		   rc_yyval.stmt = rc_stmt_create(rc_stmt_cond);
 		   rc_yyval.stmt->v.cond.node = rc_yyvsp[-2].node;
@@ -1025,7 +1033,7 @@ case 26:
 	   ;
     break;}
 case 27:
-#line 249 "rcfile.y"
+#line 258 "rcfile.y"
 {
 		   rc_yyval.stmt = rc_stmt_create(rc_stmt_cond);
 		   rc_yyval.stmt->v.cond.node = rc_yyvsp[-4].node;
@@ -1034,13 +1042,13 @@ case 27:
 	   ;
     break;}
 case 29:
-#line 259 "rcfile.y"
+#line 268 "rcfile.y"
 {
 		   rc_yyval.node = rc_yyvsp[-1].node;
 	   ;
     break;}
 case 30:
-#line 263 "rcfile.y"
+#line 272 "rcfile.y"
 {
 		   rc_yyval.node = rc_node_create(rc_node_bool);
 		   rc_yyval.node->v.bool.op = bool_and;
@@ -1049,7 +1057,7 @@ case 30:
 	   ;
     break;}
 case 31:
-#line 270 "rcfile.y"
+#line 279 "rcfile.y"
 {
 		   rc_yyval.node = rc_node_create(rc_node_bool);
 		   rc_yyval.node->v.bool.op = bool_or;
@@ -1058,7 +1066,7 @@ case 31:
 	   ;
     break;}
 case 32:
-#line 277 "rcfile.y"
+#line 286 "rcfile.y"
 {
 		   rc_yyval.node = rc_node_create(rc_node_bool);
 		   rc_yyval.node->v.bool.op = bool_not;
@@ -1067,7 +1075,7 @@ case 32:
 	   ;
     break;}
 case 35:
-#line 290 "rcfile.y"
+#line 299 "rcfile.y"
 {
 		   rc_yyval.msgpart.part = HEADER;
 		   rc_yyval.msgpart.key = rc_yyvsp[0].regex;
@@ -1075,7 +1083,7 @@ case 35:
 	   ;
     break;}
 case 36:
-#line 296 "rcfile.y"
+#line 305 "rcfile.y"
 {
 		   rc_yyval.msgpart.part = HEADER;
 		   rc_yyval.msgpart.key = NULL;
@@ -1083,21 +1091,21 @@ case 36:
 	   ;
     break;}
 case 37:
-#line 304 "rcfile.y"
+#line 313 "rcfile.y"
 {
 		   rc_yyval.msgpart.string = NULL;
 		   rc_yyval.msgpart.key = NULL;
            ;
     break;}
 case 39:
-#line 312 "rcfile.y"
+#line 321 "rcfile.y"
 {
 		   rc_yyval.msgpart = rc_yyvsp[0].msgpart;
 		   rc_yyval.msgpart.part = rc_yyvsp[-1].num;
 	   ;
     break;}
 case 41:
-#line 320 "rcfile.y"
+#line 329 "rcfile.y"
 {
 		   rc_yyval.msgpart = rc_yyvsp[0].msgpart;
 		   if (rc_yyval.msgpart.key)
@@ -1105,7 +1113,7 @@ case 41:
 	   ;
     break;}
 case 42:
-#line 328 "rcfile.y"
+#line 337 "rcfile.y"
 {
 		   rc_yyval.msgpart = rc_yyvsp[0].msgpart;
 		   if (!rc_yyval.msgpart.key) {
@@ -1115,26 +1123,26 @@ case 42:
 	   ;
     break;}
 case 43:
-#line 338 "rcfile.y"
+#line 347 "rcfile.y"
 {
 		   rc_yyval.regex = anubis_regex_compile(rc_yyvsp[-1].string, rc_yyvsp[-3].num);
 		   free(rc_yyvsp[-1].string);
 	   ;
     break;}
 case 44:
-#line 345 "rcfile.y"
+#line 354 "rcfile.y"
 {
 		   rc_yyval.string = NULL;
 	   ;
     break;}
 case 45:
-#line 349 "rcfile.y"
+#line 358 "rcfile.y"
 {
 		   rc_yyval.string = rc_yyvsp[-1].string;
 	   ;
     break;}
 case 46:
-#line 355 "rcfile.y"
+#line 364 "rcfile.y"
 {
 		   rc_yyval.node = rc_node_create(rc_node_expr);
 		   rc_yyval.node->v.expr.part = rc_yyvsp[-4].msgpart.part;
@@ -1144,7 +1152,7 @@ case 46:
 	   ;
     break;}
 case 47:
-#line 365 "rcfile.y"
+#line 374 "rcfile.y"
 {
 		   rc_yyval.num = def_regex_modifier;
 		   reg_modifier_add(&rc_yyval.num, rc_yyvsp[0].string);
@@ -1152,7 +1160,7 @@ case 47:
 	   ;
     break;}
 case 48:
-#line 371 "rcfile.y"
+#line 380 "rcfile.y"
 {
 		   reg_modifier_add(&rc_yyvsp[-1].num, rc_yyvsp[0].string);
 		   xfree(rc_yyvsp[0].string);
@@ -1160,26 +1168,26 @@ case 48:
 	   ;
     break;}
 case 49:
-#line 379 "rcfile.y"
+#line 388 "rcfile.y"
 {
 		   rc_yyval.num = def_regex_modifier;
 	   ;
     break;}
 case 51:
-#line 386 "rcfile.y"
+#line 395 "rcfile.y"
 {
 		   rc_yyval.string = rc_yyvsp[0].string;
 	   ;
     break;}
 case 52:
-#line 392 "rcfile.y"
+#line 401 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
 	   ;
     break;}
 case 55:
-#line 405 "rcfile.y"
+#line 414 "rcfile.y"
 {
 		   rc_yyval.stmt = rc_stmt_create(rc_stmt_rule);
 		   rc_yyval.stmt->v.rule.node = rc_yyvsp[-3].node;
@@ -1187,7 +1195,7 @@ case 55:
 	   ;
     break;}
 case 56:
-#line 413 "rcfile.y"
+#line 422 "rcfile.y"
 {
 		   rc_yyval.node = rc_node_create(rc_node_expr);
 		   rc_yyval.node->v.expr.part = HEADER;
@@ -1197,14 +1205,14 @@ case 56:
 	   ;
     break;}
 case 57:
-#line 423 "rcfile.y"
+#line 432 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
 	   ;
     break;}
 case 60:
-#line 434 "rcfile.y"
+#line 443 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1217,7 +1225,7 @@ case 60:
 	   ;
     break;}
 case 61:
-#line 445 "rcfile.y"
+#line 454 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1230,7 +1238,7 @@ case 61:
 	   ;
     break;}
 case 62:
-#line 456 "rcfile.y"
+#line 465 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1243,7 +1251,7 @@ case 62:
 	   ;
     break;}
 case 63:
-#line 467 "rcfile.y"
+#line 476 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1256,7 +1264,7 @@ case 63:
 	   ;
     break;}
 case 64:
-#line 478 "rcfile.y"
+#line 487 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1269,7 +1277,7 @@ case 64:
 	   ;
     break;}
 case 65:
-#line 489 "rcfile.y"
+#line 498 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1285,7 +1293,7 @@ case 65:
 	   ;
     break;}
 case 66:
-#line 505 "rcfile.y"
+#line 514 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1515,7 +1523,7 @@ rc_yyerrhandle:
     }
   return 1;
 }
-#line 513 "rcfile.y"
+#line 522 "rcfile.y"
 
 
 static int
@@ -1572,11 +1580,92 @@ rc_set_debug_level(char *arg)
 		rc_yydebug = debug_level;
 }
 
+
+/* Locations */
+
+/* To save space, each filename is allocated only once. Each filename
+   has a reference count associated with it. It is incremented
+   with each new allocation of the same string. It is decremented
+   with each attempt to free the string. Only when the reference count
+   drops to zero is the storage actually reclaimed */
+
+struct strobj {
+	char *value;          /* String value */
+	size_t refcnt;        /* Reference count */
+};
+
+/* A list of string objects */
+static LIST /* of struct strobj */ *string_list;
+
+static int
+string_comparator(void *item, void *data)
+{
+	struct strobj *s = item;
+	return strcmp (s->value, (char*) data);
+}
+
+static int
+value_comparator(void *item, void *data)
+{
+	struct strobj *s = item;
+	return s->value != data;
+}
+
+/* Looks up a string object with the given value. If not found, a
+   new object is created and added to the list. In any case the
+   reference count of the objet is incremented.
+   The return value is the string value associated with the object. */
+char *
+string_create(char *str)
+{
+	struct strobj *s = list_locate(string_list, str, string_comparator);
+	if (!s) {
+		s = xmalloc(sizeof(*s));
+		s->value = strdup(str);
+		s->refcnt = 0;
+		list_prepend(string_list, s);
+	}
+	s->refcnt++;
+	return s->value;
+}
+
+/* Destroys the object with the given string value */
+void
+string_destroy(char *str)
+{
+	struct strobj *s = list_locate(string_list, str, value_comparator);
+	if (s) {
+		if (--s->refcnt == 0) {
+			free(s->value);
+			list_remove(string_list, str, value_comparator);
+		}
+	}
+}
+
+/* Initializes LOC with the current location. If the second argument
+   is not zero, it overrides the current line number. */
+void
+rc_mark_loc(RC_LOC *loc, size_t line)
+{
+	loc->file = string_create(cfg_file);
+	loc->line = line ? line : cfg_line_num;
+}
+
+/* Reclaims the memory associated with the LOC */
+void
+rc_destroy_loc(RC_LOC *loc)
+{
+	string_destroy(loc->file);
+}
+
+
 /* Section manipulation */
+
 RC_SECTION *
-rc_section_create(char *name, RC_STMT *stmt)
+rc_section_create(char *name, size_t line, RC_STMT *stmt)
 {
 	RC_SECTION *p = xmalloc(sizeof(*p));
+	rc_mark_loc(&p->loc, line);
 	p->next = NULL;
 	p->name = name;
 	p->stmt = stmt;
@@ -1587,6 +1676,7 @@ void
 rc_section_destroy(RC_SECTION **s)
 {
 	rc_stmt_list_destroy((*s)->stmt);
+	rc_destroy_loc(&(*s)->loc);
 	xfree((*s)->name);
 	xfree(*s);
 }
@@ -1624,7 +1714,7 @@ void
 rc_section_link(RC_SECTION **ap, RC_SECTION *b)
 {
 	RC_SECTION *a, *prev;
-	
+
 	/* Remove all sections with prio == override (the default) */
 	a = *ap;
 	prev = NULL;
@@ -1682,18 +1772,11 @@ rc_section_link(RC_SECTION **ap, RC_SECTION *b)
 
 /* Assignment manipulations */
 
-static int
-_free_mem(void *item, void *data)
-{
-	free(item);
-	return 0;
-}
-
 void
 rc_asgn_destroy(RC_ASGN *asgn)
 {
 	xfree(asgn->lhs);
-	list_destroy(&asgn->rhs, _free_mem, NULL);
+	list_destroy(&asgn->rhs, anubis_free_list_item, NULL);
 }
 
 /* Bools */
@@ -1712,6 +1795,7 @@ rc_node_create(enum rc_node_type t)
 {
 	RC_NODE *p = xmalloc(sizeof(*p));
 	memset(p, 0, sizeof(*p));
+	rc_mark_loc(&p->loc, 0);
 	p->type = t;
 	return p;
 }
@@ -1730,6 +1814,7 @@ rc_node_destroy(RC_NODE *node)
 		free(node->v.expr.key);
 		anubis_regex_free(&node->v.expr.re);
 	}
+	rc_destroy_loc(&node->loc);
 	xfree(node);
 }
 
@@ -1871,6 +1956,7 @@ rc_stmt_create(enum rc_stmt_type type)
 {
 	RC_STMT *p = xmalloc(sizeof(*p));
 	memset(p, 0, sizeof(*p));
+	rc_mark_loc(&p->loc, 0);
 	p->type = type;
 	return p;
 }
@@ -1894,6 +1980,7 @@ rc_stmt_destroy(RC_STMT *stmt)
 	case rc_stmt_inst:
 		rc_inst_destroy(&stmt->v.inst);
 	}
+	rc_destroy_loc(&stmt->loc);
 	xfree(stmt);
 }
 
@@ -2053,6 +2140,7 @@ struct eval_env {
 	int refcnt;
 	char **refstr;
 	jmp_buf jmp;
+	RC_LOC loc;
 };
 
 static void asgn_eval(struct eval_env *env, RC_ASGN *asgn);
@@ -2063,6 +2151,8 @@ static void rule_eval(struct eval_env *env, RC_RULE *rule);
 static void stmt_list_eval(struct eval_env *env, RC_STMT *stmt);
 static void inst_eval(struct eval_env *env, RC_INST *inst);
 
+#define VALID_STR(s) ((s)?(s):"NULL")
+
 void
 inst_eval(struct eval_env *env, RC_INST *inst)
 {
@@ -2080,22 +2170,32 @@ inst_eval(struct eval_env *env, RC_INST *inst)
 	
 	switch (inst->opcode) {
 	case inst_stop:
+		tracefile(&env->loc, _("STOP"));
 		longjmp(env->jmp, 1);
 		break;
 
 	case inst_call:
+		tracefile(&env->loc, _("Calling %s"), inst->arg);
 		rcfile_call_section(env->method, inst->arg,
 				    env->data, env->msg);
 		break;
 		
 	case inst_add:
-		if (inst->part == BODY)
+		tracefile(&env->loc, _("ADD %s [%s] %s"),
+		      (inst->part == BODY) ? "BODY" : "HEADER",
+		      VALID_STR(inst->key2), arg);
+		if (inst->part == BODY) 
 			message_add_body(env->msg, inst->key2, arg);
 		else
 			message_add_header(env->msg, inst->key2, arg);
 		break;
 		
 	case inst_modify:
+		tracefile(&env->loc, _("MODIFY %s [%s] [%s] %s"),
+		      (inst->part == BODY) ? "BODY" : "HEADER",
+		      anubis_regex_source(inst->key), 
+		      VALID_STR(inst->key2), arg);
+		
 		if (inst->part == BODY)
 			message_modify_body(env->msg, inst->key, arg);
 		else
@@ -2104,6 +2204,8 @@ inst_eval(struct eval_env *env, RC_INST *inst)
 		break;
 		
 	case inst_remove:
+		tracefile(&env->loc, _("REMOVE HEADER [%s]"),
+		      anubis_regex_source(inst->key));
 		message_remove_headers(env->msg, inst->key);
 		break;
 		
@@ -2124,6 +2226,7 @@ asgn_eval(struct eval_env *env, RC_ASGN *asgn)
 	if (!p)
 		return;
 
+	tracefile(&env->loc, _("Executing %s"), asgn->lhs);
 	if (env->refstr) {
 		char *s;
 		LIST *arg = list_create();
@@ -2134,7 +2237,7 @@ asgn_eval(struct eval_env *env, RC_ASGN *asgn)
 		}
 		iterator_destroy(&itr);
 		p->parser(env->method, key, arg, p->data, env->data, env->msg);
-		list_destroy(&arg, _free_mem, NULL);
+		list_destroy(&arg, anubis_free_list_item, NULL);
 	} else
 		p->parser(env->method, key, asgn->rhs, p->data, env->data,
 			  env->msg);
@@ -2192,6 +2295,11 @@ expr_eval(struct eval_env *env, RC_EXPR *expr)
 	default:
 		abort();
 	}
+	if (rc)
+		tracefile(&env->loc, _("Matched condition %s[%s] \"%s\""),
+		      part_string(expr->part),
+		      VALID_STR(expr->key),
+		      anubis_regex_source(expr->re));
 	return rc;
 }
 
@@ -2200,11 +2308,13 @@ node_eval(struct eval_env *env, RC_NODE *node)
 {
 	int rc; /* It won't be used uninitialized despite what cc says.
 		   Note default: branch below */
-	
+
+	env->loc = node->loc;
 	switch (node->type) {
 	case rc_node_bool:
 		rc = bool_eval(env, &node->v.bool);
 		break;
+		
 	case rc_node_expr:
 		rc = expr_eval(env, &node->v.expr);
 		break;
@@ -2212,6 +2322,7 @@ node_eval(struct eval_env *env, RC_NODE *node)
 	default:
 		abort();
 	}
+
 	return rc;
 }
 
@@ -2256,7 +2367,9 @@ rule_eval(struct eval_env *env, RC_RULE *rule)
 void
 stmt_list_eval(struct eval_env *env, RC_STMT *stmt)
 {
-	for (; stmt; stmt = stmt->next)
+	for (; stmt; stmt = stmt->next) {
+		env->loc = stmt->loc;
+
 		switch (stmt->type) {
 		case rc_stmt_asgn:
 			asgn_eval(env, &stmt->v.asgn);
@@ -2273,6 +2386,7 @@ stmt_list_eval(struct eval_env *env, RC_STMT *stmt)
 		case rc_stmt_inst:
 			inst_eval(env, &stmt->v.inst);
 		}
+	}
 }
 
 void
@@ -2286,6 +2400,9 @@ eval_section(int method, RC_SECTION *sec, struct rc_secdef *secdef,
 	env.refstr = NULL;
 	env.msg = msg;
 	env.data = data;
+	env.loc = sec->loc;
+	
+	tracefile(&sec->loc, "Section %s", sec->name);
 	
 	if (setjmp(env.jmp) == 0)
 		stmt_list_eval(&env, sec->stmt);
@@ -2351,3 +2468,4 @@ is_prog_allowed()
 		parse_error(_("program is not allowed in this section"));
 	return p->allow_prog;
 }
+
